@@ -1,6 +1,6 @@
 # Orca
 #
-# Copyright 2005 Google Inc.
+# Copyright 2005-2006 Google Inc.
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Library General Public
@@ -25,10 +25,10 @@ ACSS definitions into engine-specific markup codes.
 
 """
 
-__id__ = "$Id: acss.py 503 2005-11-21 16:06:04Z wwalker $"
-__author__ = "$Author: wwalker $"
-__version__ = "$Revision: 503 $"
-__date__ = "$Date: 2005-11-21 11:06:04 -0500 (Mon, 21 Nov 2005) $"
+__id__ = "$Id: acss.py,v 1.5 2006/10/06 07:59:03 richb Exp $"
+__author__ = "$Author: richb $"
+__version__ = "$Revision: 1.5 $"
+__date__ = "$Date: 2006/10/06 07:59:03 $"
 __copyright__ = "Copyright (c) 2005 Google Inc."
 __license__ = "LGPL"
 
