@@ -1,6 +1,6 @@
 # Orca
 #
-# Copyright 2005 Google Inc.
+# Copyright 2005-2006 Google Inc.
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Library General Public
@@ -25,10 +25,10 @@ Dectalk voice parameter settings.
 
 """
 
-__id__ = "$Id: dectalk.py 1275 2006-08-04 15:37:25Z wwalker $"
-__author__ = "$Author: wwalker $"
-__version__ = "$Revision: 1275 $"
-__date__ = "$Date: 2006-08-04 11:37:25 -0400 (Fri, 04 Aug 2006) $"
+__id__ = "$Id: dectalk.py,v 1.8 2006/10/06 07:59:03 richb Exp $"
+__author__ = "$Author: richb $"
+__version__ = "$Revision: 1.8 $"
+__date__ = "$Date: 2006/10/06 07:59:03 $"
 __copyright__ = "Copyright (c) 2005 Google Inc."
 __license__ = "LGPL"
 
