@@ -19,9 +19,9 @@
 
 """Provides the default implementation for flat review for Orca."""
 
-__id__        = "$Id: flat_review.py 1307 2006-08-10 18:29:56Z wwalker $"
-__version__   = "$Revision: 1307 $"
-__date__      = "$Date: 2006-08-10 14:29:56 -0400 (Thu, 10 Aug 2006) $"
+__id__        = "$Id: flat_review.py,v 1.43 2006/10/10 20:17:43 wwalker Exp $"
+__version__   = "$Revision: 1.43 $"
+__date__      = "$Date: 2006/10/10 20:17:43 $"
 __copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -94,7 +94,7 @@ class Word:
         self.index = index
         self.startOffset = startOffset
         self.string = string
-        self.length = len(string)
+        self.length = len(string.decode("UTF-8"))
         self.x = x
         self.y = y
         self.width = width
@@ -156,7 +156,7 @@ class Zone:
 
         self.accessible = accessible
         self.string = string
-        self.length = len(string)
+        self.length = len(string.decode("UTF-8"))
         self.x = x
         self.y = y
         self.width = width
@@ -200,7 +200,7 @@ class Zone:
         word = None
         offset = 0
         for word in self.words:
-            nextOffset = offset + len(word.string)
+            nextOffset = offset + len(word.string.decode("UTF-8"))
             if nextOffset > charOffset:
                 return [word, charOffset - offset]
             else:
@@ -256,7 +256,7 @@ class TextZone(Zone):
             offset = self.startOffset
             for string in whitespace_re.split(self.string):
                 if len(string):
-                    endOffset = offset + len(string)
+                    endOffset = offset + len(string.decode("UTF-8"))
                     [x, y, width, height] = text.getRangeExtents(
                         offset,
                         endOffset,
@@ -346,7 +346,7 @@ class Line:
                 region.brailleOffset = brailleOffset
                 self.brailleRegions.append(region)
 
-                brailleOffset += len(region.string)
+                brailleOffset += len(region.string.decode("UTF-8"))
 
             if len(self.brailleRegions):
                 pad = braille.Region(" ")
@@ -590,7 +590,7 @@ class Context:
                 if zone.words:
                     for wordIndex in range(0, self.wordIndex):
                         regionWithFocus.cursorOffset += \
-                            len(zone.words[wordIndex].string)
+                            len(zone.words[wordIndex].string.decode("UTF-8"))
                 regionWithFocus.cursorOffset += self.charIndex
                 break
 
@@ -601,19 +601,23 @@ class Context:
         of the first relevant zone.
 
         Arguments:
-        - type: one of LINE or WINDOW
+        - type: one of ZONE, LINE or WINDOW
 
         Returns True if the locus of interest actually changed.
         """
 
-        if type == Context.LINE:
+        if (type == Context.LINE) or (type == Context.ZONE):
             lineIndex = self.lineIndex
         elif type == Context.WINDOW:
             lineIndex = 0
         else:
             raise Exception("Invalid type: %d" % type)
 
-        zoneIndex = 0
+        if type == Context.ZONE:
+            zoneIndex = self.zoneIndex
+        else:
+            zoneIndex = 0
+
         wordIndex = 0
         charIndex = 0
 
@@ -641,14 +645,18 @@ class Context:
         Returns True if the locus of interest actually changed.
         """
 
-        if type == Context.LINE:
+        if (type == Context.LINE) or (type == Context.ZONE):
             lineIndex = self.lineIndex
         elif type == Context.WINDOW:
             lineIndex  = len(self.lines) - 1
         else:
             raise Exception("Invalid type: %d" % type)
 
-        zoneIndex = len(self.lines[lineIndex].zones) - 1
+        if type == Context.ZONE:
+            zoneIndex = self.zoneIndex
+        else:
+            zoneIndex = len(self.lines[lineIndex].zones) - 1
+
         zone = self.lines[lineIndex].zones[zoneIndex]
         if zone.words:
             wordIndex = len(zone.words) - 1
