@@ -1,6 +1,6 @@
 # Orca
 #
-# Copyright 2005 Google Inc.
+# Copyright 2005-2006 Google Inc.
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Library General Public
@@ -25,10 +25,10 @@ Outloud voice parameter settings.
 
 """
 
-__id__ = "$Id: outloud.py,v 1.5 2006/07/31 12:41:30 wwalker Exp $"
-__author__ = "$Author: wwalker $"
-__version__ = "$Revision: 1.5 $"
-__date__ = "$Date: 2006/07/31 12:41:30 $"
+__id__ = "$Id: outloud.py,v 1.6 2006/10/06 07:59:03 richb Exp $"
+__author__ = "$Author: richb $"
+__version__ = "$Revision: 1.6 $"
+__date__ = "$Date: 2006/10/06 07:59:03 $"
 __copyright__ = "Copyright (c) 2005 Google Inc."
 __license__ = "LGPL"
 
