@@ -19,9 +19,9 @@
 
 """Custom script for StarOffice and OpenOffice."""
 
-__id__        = "$Id: StarOffice.py,v 1.47 2006/10/03 18:52:11 wwalker Exp $"
-__version__   = "$Revision: 1.47 $"
-__date__      = "$Date: 2006/10/03 18:52:11 $"
+__id__        = "$Id: StarOffice.py,v 1.56 2006/10/14 15:17:52 wwalker Exp $"
+__version__   = "$Revision: 1.56 $"
+__date__      = "$Date: 2006/10/14 15:17:52 $"
 __copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -43,6 +43,67 @@ from orca.orca_i18n import _ # for gettext support
 
 inputLineForCell = None
 
+# Dictionaries for the calc dynamic row and column headers.
+#
+dynamicColumnHeaders = {}
+dynamicRowHeaders = {}
+
+def getCalcTable(obj):
+    """Get the table that this table cell is in.
+
+    Arguments:
+    - obj: the table cell.
+
+    Return the table that this table cell is in, or None if this object
+    isn't in a table.
+    """
+
+    table = None
+    if obj.role == rolenames.ROLE_TABLE_CELL and obj.parent:
+        table = obj.parent.table
+
+    return table
+
+def getDynamicColumnHeaderCell(obj, column):
+    """Given a table cell, return the dynamic column header cell associated
+    with it.
+
+    Arguments:
+    - obj: the table cell.
+    - column: the column that this dynamic header is on.
+
+    Return the dynamic column header cell associated with the given table cell.
+    """
+
+    accCell = None
+    parent = obj.parent
+    if parent and parent.table:
+        row = parent.table.getRowAtIndex(obj.index)
+        cell = parent.table.getAccessibleAt(row, column)
+        accCell = atspi.Accessible.makeAccessible(cell)
+
+    return accCell
+
+def getDynamicRowHeaderCell(obj, row):
+    """Given a table cell, return the dynamic row header cell associated
+    with it.
+
+    Arguments:
+    - obj: the table cell.
+    - row: the row that this dynamic header is on.
+
+    Return the dynamic row header cell associated with the given table cell.
+    """
+
+    accCell = None
+    parent = obj.parent
+    if parent and parent.table:
+        column = parent.table.getColumnAtIndex(obj.index)
+        cell = parent.table.getAccessibleAt(row, column)
+        accCell = atspi.Accessible.makeAccessible(cell)
+
+    return accCell
+
 def locateInputLine(obj):
     """Return the spread sheet input line. This only needs to be found
     the very first time a spread sheet table cell gets focus. We use the
@@ -93,11 +154,121 @@ def isSpreadSheetCell(obj):
     return util.isDesiredFocusedItem(obj, rolesList)
 
 class BrailleGenerator(braillegenerator.BrailleGenerator):
-    """Overrides _getBrailleRegionsForTableCell so that, when we are in
+    """Overrides _getBrailleRegionsForTableCellRow so that , when we are
+    in a spread sheet, we can braille the dynamic row and column headers
+    (assuming they are set).
+    Overrides _getBrailleRegionsForTableCell so that, when we are in
     a spread sheet, we can braille the location of the table cell as well
     as the contents.
     """
 
+    def __init__(self, script):
+        braillegenerator.BrailleGenerator.__init__(self, script)
+
+    def _getBrailleRegionsForTableCellRow(self, obj):
+        """Get the braille for a table cell row or a single table cell
+        if settings.readTableCellRow is False.
+
+        Arguments:
+        - obj: the table cell
+
+        Returns a list where the first element is a list of Regions to display
+        and the second element is the Region which should get focus.
+        """
+
+        global dynamicColumnHeaders, dynamicRowHeaders
+        global getColumnHeaderCell, getRowHeaderCell, isSpreadSheetCell
+
+        regions = []
+
+        if isSpreadSheetCell(obj):
+
+            # Check to see if this spread sheet cell has either a dynamic
+            # column heading or row heading (or both) associated with it. 
+            # If it does, then braille those first before brailling the 
+            # cell contents.
+            #
+            table = getCalcTable(obj)
+            parent = obj.parent
+
+            if parent.__dict__.has_key("lastColumn") and \
+               parent.lastColumn != parent.table.getColumnAtIndex(obj.index):
+                if dynamicColumnHeaders.has_key(table):
+                    row = dynamicColumnHeaders[table]
+                    header = getDynamicRowHeaderCell(obj, row)
+                    if header.text:
+                        text = self._script.getText(header, 0, -1)
+                        if text:
+                            regions.append(braille.Region(" " + text + " "))
+
+            if parent.__dict__.has_key("lastRow") and \
+               parent.lastRow != parent.table.getRowAtIndex(obj.index):
+                if dynamicRowHeaders.has_key(table):
+                    column = dynamicRowHeaders[table]
+                    header = getDynamicColumnHeaderCell(obj, column)
+                    if header.text:
+                        text = self._script.getText(header, 0, -1)
+                        if text:
+                            regions.append(braille.Region(" " + text + " "))
+
+            # Adding in a check here to make sure that the parent is a
+            # valid table. It's possible that the parent could be a
+            # table cell too (see bug #351501).
+            #
+            if settings.readTableCellRow and obj.parent.table:
+                rowRegions = []
+                savedBrailleVerbosityLevel = settings.brailleVerbosityLevel
+                settings.brailleVerbosityLevel = \
+                                             settings.VERBOSITY_LEVEL_BRIEF
+
+                parent = obj.parent
+                row = parent.table.getRowAtIndex(obj.index)
+                column = parent.table.getColumnAtIndex(obj.index)
+
+                # This is an indication of whether we should speak all the
+                # table cells (the user has moved focus up or down a row),
+                # or just the current one (focus has moved left or right in
+                # the same row).
+                #
+                speakAll = True
+                if parent.__dict__.has_key("lastRow") and \
+                    parent.__dict__.has_key("lastColumn"):
+                    speakAll = (parent.lastRow != row) or \
+                           ((row == 0 or row == parent.table.nRows-1) and \
+                            parent.lastColumn == column)
+
+                if speakAll:
+                    focusRowRegion = None
+                    for i in range(0, parent.table.nColumns):
+                        accRow = parent.table.getAccessibleAt(row, i)
+                        cell = atspi.Accessible.makeAccessible(accRow)
+                        showing = cell.state.count( \
+                                        atspi.Accessibility.STATE_SHOWING)
+                        if showing:
+                            [cellRegions, focusRegion] = \
+                                self._getBrailleRegionsForTableCell(cell)
+                            if len(rowRegions):
+                                rowRegions.append(braille.Region(" "))
+                            rowRegions.append(cellRegions[0])
+                            if i == column:
+                                focusRowRegion = cellRegions[0]
+                    regions.extend(rowRegions)
+                    settings.brailleVerbosityLevel = savedBrailleVerbosityLevel
+                else:
+                    [cellRegions, focusRegion] = \
+                                self._getBrailleRegionsForTableCell(obj)
+                    regions.extend(cellRegions)
+            else:
+                [cellRegions, focusRegion] = \
+                                self._getBrailleRegionsForTableCell(obj)
+                regions.extend(cellRegions)
+            regions = [regions, focusRegion]
+        else:
+            brailleGen = braillegenerator.BrailleGenerator
+            regions = brailleGen._getBrailleRegionsForTableCellRow(self, obj)
+
+        return regions
+
     def _getBrailleRegionsForTableCell(self, obj):
         """Get the braille for a table cell. If this isn't inside a
         spread sheet, just return the regions returned by the default
@@ -116,8 +287,8 @@ class BrailleGenerator(braillegenerator.BrailleGenerator):
             if inputLineForCell == None:
                 inputLineForCell = locateInputLine(obj)
 
-            text = util.getDisplayedText(obj)
             regions = []
+            text = util.getDisplayedText(obj)
             componentRegion = braille.Component(obj, text)
             regions.append(componentRegion)
 
@@ -127,7 +298,7 @@ class BrailleGenerator(braillegenerator.BrailleGenerator):
             # have already done this for us.
             #
             if obj.text:
-                objectText = obj.text.getText(0, -1)
+                objectText = self._script.getText(obj, 0, -1)
                 if objectText and len(objectText) != 0:
                     regions.append(braille.Region(" " + obj.name))
 
@@ -137,13 +308,107 @@ class BrailleGenerator(braillegenerator.BrailleGenerator):
             brailleGen = braillegenerator.BrailleGenerator
             regions = brailleGen._getBrailleRegionsForTableCell(self, obj)
 
-            return regions
+        return regions
 
 class SpeechGenerator(speechgenerator.SpeechGenerator):
-    """Overrides _getSpeechForTableCell so that, when we are in a spread
+    """Overrides _getSpeechForTableCellRow so that , when we are in a
+    spread sheet, we can speak the dynamic row and column headers
+    (assuming they are set).
+    Overrides _getSpeechForTableCell so that, when we are in a spread
     sheet, we can speak the location of the table cell as well as the
     contents.
     """
+    def __init__(self, script):
+        speechgenerator.SpeechGenerator.__init__(self, script)
+
+    def _getSpeechForTableCellRow(self, obj, already_focused):
+        """Get the speech for a table cell row or a single table cell
+        if settings.readTableCellRow is False. If this isn't inside a
+        spread sheet, just return the utterances returned by the default
+        table cell speech handler.
+
+        Arguments:
+        - obj: the table cell
+        - already_focused: False if object just received focus
+
+        Returns a list of utterances to be spoken for the object.
+        """
+
+        global dynamicColumnHeaders, dynamicRowHeaders
+        global getColumnHeaderCell, getRowHeaderCell, isSpreadSheetCell
+
+        utterances = []
+
+        if isSpreadSheetCell(obj):
+            if (not already_focused):
+
+                # Check to see if this spread sheet cell has either a dynamic
+                # column heading or row heading (or both) associated with it.
+                # If it does, then speak those first before speaking the cell
+                # contents.
+                #
+                table = getCalcTable(obj)
+                parent = obj.parent
+
+                if parent.__dict__.has_key("lastColumn") and \
+                   parent.lastColumn != \
+                   parent.table.getColumnAtIndex(obj.index):
+                    if dynamicColumnHeaders.has_key(table):
+                        row = dynamicColumnHeaders[table]
+                        header = getDynamicRowHeaderCell(obj, row)
+                        if header.text:
+                            text = self._script.getText(header, 0, -1)
+                            if text:
+                                utterances.append(text)
+
+                if parent.__dict__.has_key("lastRow") and \
+                   parent.lastRow != parent.table.getRowAtIndex(obj.index):
+                    if dynamicRowHeaders.has_key(table):
+                        column = dynamicRowHeaders[table]
+                        header = getDynamicColumnHeaderCell(obj, column)
+                        if header.text:
+                            text = self._script.getText(header, 0, -1)
+                            if text:
+                                utterances.append(text)
+
+                if settings.readTableCellRow:
+                    parent = obj.parent
+                    row = parent.table.getRowAtIndex(obj.index)
+                    column = parent.table.getColumnAtIndex(obj.index)
+
+                    # This is an indication of whether we should speak all the
+                    # table cells (the user has moved focus up or down a row),
+                    # or just the current one (focus has moved left or right in
+                    # the same row).
+                    #
+                    speakAll = True
+                    if parent.__dict__.has_key("lastRow") and \
+                        parent.__dict__.has_key("lastColumn"):
+                        speakAll = (parent.lastRow != row) or \
+                               ((row == 0 or row == parent.table.nRows-1) and \
+                                parent.lastColumn == column)
+
+                    if speakAll:
+                        for i in range(0, parent.table.nColumns):
+                            accRow = parent.table.getAccessibleAt(row, i)
+                            cell = atspi.Accessible.makeAccessible(accRow)
+                            showing = cell.state.count( \
+                                          atspi.Accessibility.STATE_SHOWING)
+                            if showing:
+                                utterances.extend(self._getSpeechForTableCell(\
+                                                  cell, already_focused))
+                    else:
+                        utterances.extend(self._getSpeechForTableCell(obj,
+                                                             already_focused))
+                else:
+                    utterances.extend(self._getSpeechForTableCell(obj,
+                                                             already_focused))
+        else:
+            speechGen = speechgenerator.SpeechGenerator
+            utterances = speechGen._getSpeechForTableCellRow(self, obj,
+                                                             already_focused)
+
+        return utterances
 
     def _getSpeechForTableCell(self, obj, already_focused):
         """Get the speech for a table cell. If this isn't inside a
@@ -161,6 +426,7 @@ class SpeechGenerator(speechgenerator.SpeechGenerator):
 
         if isSpreadSheetCell(obj):
             utterances = []
+
             utterances.append(util.getDisplayedText(\
                     util.getRealActiveDescendant(obj)))
 
@@ -173,7 +439,7 @@ class SpeechGenerator(speechgenerator.SpeechGenerator):
             # have already done this for us.
             #
             if obj.text:
-                objectText = obj.text.getText(0, -1)
+                objectText = self._script.getText(obj, 0, -1)
                 if objectText and len(objectText) != 0:
                     utterances.append(" " + obj.name)
         else:
@@ -217,17 +483,22 @@ class Script(default.Script):
         self.lastStartOff = -1
         self.lastEndOff = -1
 
+        # Used to determine if the user has double-clicked the dynamic
+        # row/column hotkeys.
+
+        self.lastDynamicEvent = None
+
     def getBrailleGenerator(self):
         """Returns the braille generator for this script.
         """
 
-        return BrailleGenerator()
+        return BrailleGenerator(self)
 
     def getSpeechGenerator(self):
         """Returns the speech generator for this script.
         """
 
-        return SpeechGenerator()
+        return SpeechGenerator(self)
 
     def setupInputEventHandlers(self):
         """Defines InputEventHandler fields for this script that can be
@@ -243,6 +514,16 @@ class Script(default.Script):
                 Script.speakInputLine,
                 _("Speaks the contents of the input line."))
 
+        self.inputEventHandlers["setDynamicColumnHeadersHandler"] = \
+            input_event.InputEventHandler(
+                Script.setDynamicColumnHeaders,
+                _("Set the row to use as dynamic column headers when speaking calc cells."))
+
+        self.inputEventHandlers["setDynamicRowHeadersHandler"] = \
+            input_event.InputEventHandler(
+                Script.setDynamicRowHeaders,
+                _("Set the column to use as dynamic row headers to use when speaking calc cells."))
+
     def getKeyBindings(self):
         """Defines the key bindings for this script. Setup the default
         key bindings, then add one in for reading the input line.
@@ -259,6 +540,20 @@ class Script(default.Script):
                 1 << settings.MODIFIER_ORCA,
                 self.inputEventHandlers["speakInputLineHandler"]))
 
+        keyBindings.add(
+            keybindings.KeyBinding(
+                "r",
+                1 << settings.MODIFIER_ORCA,
+                1 << settings.MODIFIER_ORCA,
+                self.inputEventHandlers["setDynamicColumnHeadersHandler"]))
+
+        keyBindings.add(
+            keybindings.KeyBinding(
+                "c",
+                1 << settings.MODIFIER_ORCA,
+                1 << settings.MODIFIER_ORCA,
+                self.inputEventHandlers["setDynamicRowHeadersHandler"]))
+
         return keyBindings
 
     def speakInputLine(self, inputEvent):
@@ -286,6 +581,111 @@ class Script(default.Script):
                         "StarOffice.speakInputLine: contents: %s" % inputLine)
                 speech.speak(inputLine)
 
+    def getCalcRow(self, cell):
+        """Get the row number in the table that this table cell is on.
+
+        Arguments:
+        - cell: the table cell to get the row number for.
+
+        Return the row number that this table cell is on, or None if
+        this isn't a table cell.
+        """
+
+        row = None
+        if cell.role == rolenames.ROLE_TABLE_CELL:
+            parent = cell.parent
+            if parent and parent.table:
+                row = parent.table.getRowAtIndex(cell.index)
+
+        return row
+
+    def getCalcColumn(self, cell):
+        """Get the column number in the table that this table cell is on.
+
+        Arguments:
+        - cell: the table cell to get the column number for.
+
+        Return the column number that this table cell is on, or None if
+        this isn't a table cell.
+        """
+
+        column = None
+        if cell.role == rolenames.ROLE_TABLE_CELL:
+            parent = cell.parent
+            if parent and parent.table:
+                column = parent.table.getColumnAtIndex(cell.index)
+
+        return column
+
+    def setDynamicColumnHeaders(self, inputEvent):
+        """Set the row for the dynamic header columns to use when speaking 
+        calc cell entries. In order to set the row, the user should first set
+        focus to the row that they wish to define and then press Insert-r.
+
+        Once the user has defined the row, it will be used to first speak
+        this header when moving between columns.
+
+        A "double-click" of the Insert-c hotkey, will clear the dynamic
+        header column.
+
+        Arguments:
+        - inputEvent: if not None, the input event that caused this action.
+        """
+
+        global dynamicColumnHeaders, getCalcTable
+
+        debug.println(self.debugLevel, "StarOffice.setDynamicColumnHeaders.")
+
+        clickCount = util.getClickCount(self.lastDynamicEvent, inputEvent)
+        table = getCalcTable(orca_state.locusOfFocus)
+        if table:
+            row = self.getCalcRow(orca_state.locusOfFocus)
+            if clickCount == 2:
+                try:
+                    del dynamicColumnHeaders[table]
+                except:
+                    pass
+            else:
+                dynamicColumnHeaders[table] = row
+        self.lastDynamicEvent = inputEvent
+
+        return True
+
+    def setDynamicRowHeaders(self, inputEvent):
+        """Set the column for the dynamic header rows to use when speaking 
+        calc cell entries. In order to set the column, the user should first 
+        set focus to the column that they wish to define and then press 
+        Insert-c.
+
+        Once the user has defined the column, it will be used to first speak
+        this header when moving between rows.
+
+        A "double-click" of the Insert-r hotkey, will clear the dynamic
+        header row.
+
+        Arguments:
+        - inputEvent: if not None, the input event that caused this action.
+        """
+
+        global dynamicRowHeaders, getCalcTable
+
+        debug.println(self.debugLevel, "StarOffice.setDynamicRowHeaders.")
+
+        clickCount = util.getClickCount(self.lastDynamicEvent, inputEvent)
+        table = getCalcTable(orca_state.locusOfFocus)
+        if table:
+            column = self.getCalcColumn(orca_state.locusOfFocus)
+            if clickCount == 2:
+                try:
+                    del dynamicRowHeaders[table]
+                except:
+                    pass
+            else:
+                dynamicRowHeaders[table] = column
+        self.lastDynamicEvent = inputEvent
+
+        return True
+
     def readMisspeltWord(self, event, pane):
         """Speak/braille the current misspelt word plus its context.
            The spell check dialog contains a "paragraph" which shows the
@@ -457,7 +857,6 @@ class Script(default.Script):
     # 1) Writer: text paragraph.
     # 2) Writer: spell checking dialog.
     # 3) Welcome to StarOffice dialog.
-    # 4) Calc - spread sheet table.
 
     def locusOfFocusChanged(self, event, oldLocusOfFocus, newLocusOfFocus):
         """Called when the visual object with focus changes.
@@ -479,9 +878,16 @@ class Script(default.Script):
 
         # 1) Writer: text paragraph.
         #
-        # When the focus is on a paragraph in the Document view of the Writer,
-        # then just speak/braille the current line (rather than speaking a
-        # bogus initial "paragraph" utterance as well).
+        # We need to handle two things here:
+        #
+        # If the old locus of focus was on the File->New->Text Document
+        # menu item and we are currently have focus on an empty text
+        # paragraph, then we've just created the first new text document
+        # in Writer. Announce it by doing a "where am I".
+        #
+        # Also, when the focus is on a paragraph in the Document view of 
+        # the Writer, then just speak/braille the current line (rather than 
+        # speaking a bogus initial "paragraph" utterance as well).
 
         rolesList = [rolenames.ROLE_PARAGRAPH, \
                      rolenames.ROLE_UNKNOWN, \
@@ -494,6 +900,12 @@ class Script(default.Script):
                   "StarOffice.locusOfFocusChanged - Writer: text paragraph.")
 
             result = util.getTextLineAtCaret(event.source)
+            result[0] = result[0].decode("UTF-8")
+
+            if oldLocusOfFocus.role == rolenames.ROLE_MENU_ITEM and \
+               oldLocusOfFocus.name == _("Text Document") and \
+               len(result[0]) == 0:
+                self.whereAmI(None)
 
             # Check to see if there are any hypertext links in this paragraph.
             # If no, then just speak the whole line. Otherwise, split the
@@ -503,8 +915,9 @@ class Script(default.Script):
             hypertext = event.source.hypertext
             if not hypertext or (hypertext.getNLinks() == 0):
                 if settings.enableSpeechIndentation:
-                    self.speakTextIndentation(event.source, result[0])
-                speech.speak(result[0], None, False)
+                    self.speakTextIndentation(event.source,
+                                              result[0].encode("UTF-8"))
+                speech.speak(result[0].encode("UTF-8"), None, False)
             else:
                 started = False
                 startOffset = 0
@@ -513,7 +926,7 @@ class Script(default.Script):
                         if started:
                             endOffset = i
                             self.sayWriterWord(event.source,
-                                result[0][startOffset:endOffset+1],
+                                result[0][startOffset:endOffset+1].encode("UTF-8"),
                                 startOffset, endOffset)
                             startOffset = i
                             started = False
@@ -525,8 +938,8 @@ class Script(default.Script):
                 if started:
                     endOffset = len(result[0])
                     self.sayWriterWord(event.source,
-                                       result[0][startOffset:endOffset],
-                                       startOffset, endOffset)
+                        result[0][startOffset:endOffset].encode("UTF-8"),
+                        startOffset, endOffset)
 
             braille.displayRegions(brailleGen.getBrailleRegions(event.source))
 
@@ -648,73 +1061,6 @@ class Script(default.Script):
                     + "Registration: Register Now radio button.")
                 self.handleSetupPanel(event.source.parent)
 
-        # 4) Calc - spread sheet table.
-        #
-        # If we are in the Calc spread sheet table, and we want to read all
-        # of the row (as opposed to a single cell), then generate the braille
-        # and speech for each of the showing spread sheet cells in the current
-        # row.
-        #
-        rolesList = [rolenames.ROLE_TABLE, \
-                     rolenames.ROLE_UNKNOWN, \
-                     rolenames.ROLE_SCROLL_PANE, \
-                     rolenames.ROLE_PANEL, \
-                     rolenames.ROLE_ROOT_PANE, \
-                     rolenames.ROLE_FRAME, \
-                     rolenames.ROLE_APPLICATION]
-        if settings.readTableCellRow and \
-           util.isDesiredFocusedItem(event.source, rolesList):
-            debug.println(self.debugLevel, "StarOffice.locusOfFocusChanged - " \
-                          + "Calc: spread sheet table - read row.")
-            table = event.source.table
-            row = table.getRowAtIndex(newLocusOfFocus.index)
-            column = table.getColumnAtIndex(newLocusOfFocus.index)
-
-            # This is an indication of whether we should speak all the
-            # table cells (the user has moved focus up or down a row),
-            # or just the current one (focus has moved left or right in
-            # the same row).
-            #
-            speakAll = True
-            if event.source.__dict__.has_key("lastRow") and \
-                event.source.__dict__.has_key("lastColumn"):
-                speakAll = (event.source.lastRow != row) or \
-                       ((row == 0 or row == event.source.table.nRows-1) and \
-                        event.source.lastColumn == column)
-
-            if speakAll:
-                utterances = []
-                regions = []
-
-                # [[[TODO: richb - one speedup improvement we can do here is
-                # to look left and look right along the row from the current
-                # column, to just get the range of cells to show.]]]
-                #
-                for i in range(0, table.nColumns):
-                    obj = table.getAccessibleAt(row, i)
-                    cell = atspi.Accessible.makeAccessible(obj)
-
-                    showing = cell.state.count(atspi.Accessibility.STATE_SHOWING)
-                    while cell.childCount:
-                        cell = cell.child(0)
-
-                    if showing:
-                        [cellRegions, focusedRegion] = \
-                            brailleGen.getBrailleRegions(cell)
-                        regions.extend(cellRegions)
-                        regions.append(braille.Region(" "))
-                        if column == i:
-                            cellWithFocus = focusedRegion
-
-                        result = speechGen.getSpeech(cell, True)
-                        utterances.append(result[0])
-
-                braille.displayRegions([regions, cellWithFocus])
-                speech.speakUtterances(utterances)
-                event.source.lastColumn = column
-                event.source.lastRow = row
-                return
-
         # Pass the event onto the parent class to be handled in the default way.
 
         default.Script.locusOfFocusChanged(self, event,
@@ -857,7 +1203,7 @@ class Script(default.Script):
 
                     if cell.text:
                         cellText = self.getText(cell, 0, -1)
-                        if cellText and len(cellText) != 0:
+                        if cellText and len(cellText):
                             if inputLineForCell and inputLineForCell.text:
                                 inputLine = self.getText(inputLineForCell,0,-1)
                                 if inputLine and \
@@ -891,10 +1237,12 @@ class Script(default.Script):
         - startOffset: the starting character position
         - endOffset: the ending character position
         """
-        text = unicode(obj.text.getText(0, -1))
+        text = obj.text.getText(0, -1).decode("UTF-8")
         if startOffset >= len(text):
             startOffset = len(text) - 1
-        if startOffset >= endOffset:
+        if endOffset == -1:
+            endOffset = len(text)
+        elif startOffset >= endOffset:
             endOffset = startOffset + 1
         string = text[max(0,startOffset):min(len(text),endOffset)]
         string = string.encode("UTF-8")
